@@ -66,10 +66,10 @@ class HTMLElement(object):
 
         :param attribute: The name of the attribute.
         '''
-        return self.marionette._send_message('getElementAttribute', 'value', element=self.id, name=attribute)
+        return self.marionette._send_message('getElementAttribute', 'value', id=self.id, name=attribute)
 
     def click(self):
-        return self.marionette._send_message('clickElement', 'ok', element=self.id)
+        return self.marionette._send_message('clickElement', 'ok', id=self.id)
 
     def tap(self, x=None, y=None):
         '''
@@ -80,14 +80,14 @@ class HTMLElement(object):
         :param x: X-coordinate of tap event. If not given, default to the
          center of the element.
         '''
-        return self.marionette._send_message('singleTap', 'ok', element=self.id, x=x, y=y)
+        return self.marionette._send_message('singleTap', 'ok', id=self.id, x=x, y=y)
 
     @property
     def text(self):
         '''
         Returns the visible text of the element, and its child elements.
         '''
-        return self.marionette._send_message('getElementText', 'value', element=self.id)
+        return self.marionette._send_message('getElementText', 'value', id=self.id)
 
     def send_keys(self, *string):
         '''
@@ -104,52 +104,52 @@ class HTMLElement(object):
             else:
                 for i in range(len(val)):
                     typing.append(val[i])
-        return self.marionette._send_message('sendKeysToElement', 'ok', element=self.id, value=typing)
+        return self.marionette._send_message('sendKeysToElement', 'ok', id=self.id, value=typing)
 
     def clear(self):
         '''
         Clears the input of the element.
         '''
-        return self.marionette._send_message('clearElement', 'ok', element=self.id)
+        return self.marionette._send_message('clearElement', 'ok', id=self.id)
 
     def is_selected(self):
         '''
         Returns True if the element is selected.
         '''
-        return self.marionette._send_message('isElementSelected', 'value', element=self.id)
+        return self.marionette._send_message('isElementSelected', 'value', id=self.id)
 
     def is_enabled(self):
         '''
         Returns True if the element is enabled.
         '''
-        return self.marionette._send_message('isElementEnabled', 'value', element=self.id)
+        return self.marionette._send_message('isElementEnabled', 'value', id=self.id)
 
     def is_displayed(self):
         '''
         Returns True if the element is displayed.
         '''
-        return self.marionette._send_message('isElementDisplayed', 'value', element=self.id)
+        return self.marionette._send_message('isElementDisplayed', 'value', id=self.id)
 
     @property
     def size(self):
         '''
         A dictionary with the size of the element.
         '''
-        return self.marionette._send_message('getElementSize', 'value', element=self.id)
+        return self.marionette._send_message('getElementSize', 'value', id=self.id)
 
     @property
     def tag_name(self):
         '''
         The tag name of the element.
         '''
-        return self.marionette._send_message('getElementTagName', 'value', element=self.id)
+        return self.marionette._send_message('getElementTagName', 'value', id=self.id)
 
     @property
     def location(self):
         '''
         A dictionary with the x and y location of an element
         '''
-        return self.marionette._send_message('getElementPosition', 'value', element=self.id)
+        return self.marionette._send_message('getElementPosition', 'value', id=self.id)
 
     def value_of_css_property(self, property_name):
         '''
@@ -158,8 +158,13 @@ class HTMLElement(object):
         :param property_name: Property name to get the value of.
         '''
         return self.marionette._send_message('getElementValueOfCssProperty', 'value',
-                                             element=self.id,
+                                             id=self.id,
                                              propertyName=property_name)
+    def submit(self):
+        '''
+        Submits if the element is a form or is within a form
+        '''
+        return self.marionette._send_message('submitElement', 'ok', id=self.id)
 
 class Actions(object):
     '''
@@ -421,7 +426,8 @@ class Marionette(object):
                  profile=None, emulator=None, sdcard=None, emulatorBinary=None,
                  emulatorImg=None, emulator_res=None, gecko_path=None,
                  connectToRunningEmulator=False, homedir=None, baseurl=None,
-                 noWindow=False, logcat_dir=None, busybox=None, symbols_path=None, timeout=None):
+                 noWindow=False, logcat_dir=None, busybox=None, symbols_path=None,
+                 timeout=None, device_serial=None):
         self.host = host
         self.port = self.local_port = port
         self.bin = bin
@@ -438,6 +444,7 @@ class Marionette(object):
         self._test_name = None
         self.symbols_path = symbols_path
         self.timeout = timeout
+        self.device_serial = device_serial
 
         if bin:
             port = int(self.port)
@@ -525,7 +532,7 @@ class Marionette(object):
             # flagging the error.
             sys.exit()
 
-    def wait_for_port(self, timeout=3000):
+    def wait_for_port(self, timeout=30):
         starttime = datetime.datetime.now()
         while datetime.datetime.now() - starttime < datetime.timedelta(seconds=timeout):
             try:
@@ -545,11 +552,11 @@ class Marionette(object):
         if not self.session and command not in ('newSession', 'getStatus'):
             raise MarionetteException(message="Please start a session")
 
-        message = { 'type': command }
+        message = { 'name': command }
         if self.session:
-            message['session'] = self.session
+            message['sessionId'] = self.session
         if kwargs:
-            message.update(kwargs)
+            message['parameters'] = kwargs
 
         try:
             response = self.client.send(message)
@@ -576,7 +583,7 @@ class Marionette(object):
                                       "command against.")
         cmd = cmd.encode("ascii")
         result = self.emulator._run_telnet(cmd)
-        return self.client.send({"type": "emulatorCmdResult",
+        return self.client.send({"name": "emulatorCmdResult",
                                  "id": response.get("id"),
                                  "result": result})
 
@@ -714,7 +721,7 @@ class Marionette(object):
         :param timeout: The maximum number of milliseconds an asynchronous
          script can run without causing an ScriptTimeoutException to be raised
         '''
-        response = self._send_message('setScriptTimeout', 'ok', value=timeout)
+        response = self._send_message('setScriptTimeout', 'ok', ms=timeout)
         return response
 
     def set_search_timeout(self, timeout):
@@ -730,7 +737,7 @@ class Marionette(object):
 
         :param timeout: Timeout in milliseconds.
         '''
-        response = self._send_message('setSearchTimeout', 'ok', value=timeout)
+        response = self._send_message('setSearchTimeout', 'ok', ms=timeout)
         return response
 
     @property
@@ -801,7 +808,7 @@ class Marionette(object):
 
         :param window_id: The id or name of the window to switch to.
         '''
-        response = self._send_message('switchToWindow', 'ok', value=window_id)
+        response = self._send_message('switchToWindow', 'ok', name=window_id)
         self.window = window_id
         return response
 
@@ -814,6 +821,12 @@ class Marionette(object):
             return HTMLElement(self, response)
         return None
 
+    def switch_to_default_content(self):
+        '''
+        Switch the current context to page's default content.
+        '''
+        return self.switch_to_frame()
+
     def switch_to_frame(self, frame=None, focus=True):
         '''
         Switch the current context to the specified frame. Subsequent commands will operate in the context of the specified frame, if applicable.
@@ -824,7 +837,7 @@ class Marionette(object):
         if isinstance(frame, HTMLElement):
             response = self._send_message('switchToFrame', 'ok', element=frame.id, focus=focus)
         else:
-            response = self._send_message('switchToFrame', 'ok', value=frame, focus=focus)
+            response = self._send_message('switchToFrame', 'ok', id=frame, focus=focus)
         return response
 
     def get_url(self):
@@ -850,12 +863,12 @@ class Marionette(object):
 
         :param url: The url to navigate to.
         '''
-        response = self._send_message('goUrl', 'ok', value=url)
+        response = self._send_message('goUrl', 'ok', url=url)
         return response
 
     def timeouts(self, timeout_type, ms):
         assert(timeout_type == self.TIMEOUT_SEARCH or timeout_type == self.TIMEOUT_SCRIPT or timeout_type == self.TIMEOUT_PAGE)
-        response = self._send_message('timeouts', 'ok', timeoutType=timeout_type, ms=ms)
+        response = self._send_message('timeouts', 'ok', type=timeout_type, ms=ms)
         return response
 
     def go_back(self):
@@ -922,7 +935,7 @@ class Marionette(object):
         args = self.wrapArguments(script_args)
         response = self._send_message('executeJSScript',
                                       'value',
-                                      value=script,
+                                      script=script,
                                       args=args,
                                       async=async,
                                       newSandbox=new_sandbox,
@@ -1003,7 +1016,7 @@ class Marionette(object):
         frame = stack[-2:-1][0] # grab the second-to-last frame
         response = self._send_message('executeScript',
                                       'value',
-                                      value=script,
+                                      script=script,
                                       args=args,
                                       newSandbox=new_sandbox,
                                       specialPowers=special_powers,
@@ -1051,7 +1064,7 @@ class Marionette(object):
         frame = stack[-2:-1][0] # grab the second-to-last frame
         response = self._send_message('executeAsyncScript',
                                       'value',
-                                      value=script,
+                                      script=script,
                                       args=args,
                                       newSandbox=new_sandbox,
                                       specialPowers=special_powers,
@@ -1085,7 +1098,7 @@ class Marionette(object):
         if id:
             kwargs['element'] = id
         response = self._send_message('findElement', 'value', **kwargs)
-        element = HTMLElement(self, response)
+        element = HTMLElement(self, response['ELEMENT'])
         return element
 
     def find_elements(self, method, target, id=None):
@@ -1258,4 +1271,4 @@ class Marionette(object):
         lights = None
         if highlights is not None:
             lights = [highlight.id for highlight in highlights if highlights]
-        return self._send_message("screenShot", 'value', element=element, highlights=lights)
+        return self._send_message("screenShot", 'value', id=element, highlights=lights)
