@@ -11,6 +11,7 @@ import socket
 import sys
 import time
 import traceback
+import random
 import moznetwork
 import xml.dom.minidom as dom
 
@@ -18,30 +19,94 @@ from manifestparser import TestManifest
 from mozhttpd import MozHttpd
 
 from marionette import Marionette
+from moztest.results import TestResultCollection
 from marionette_test import MarionetteJSTestCase, MarionetteTestCase
 
 
-class MarionetteTestResult(unittest._TextTestResult):
+class MarionetteTestResult(unittest._TextTestResult, TestResultCollection):
 
     def __init__(self, *args, **kwargs):
-        self.marionette = kwargs['marionette']
-        del kwargs['marionette']
-        super(MarionetteTestResult, self).__init__(*args, **kwargs)
+        self.marionette = kwargs.pop('marionette')
+        TestResultCollection.__init__(self, 'MarionetteTest')
+        unittest._TextTestResult.__init__(self, *args, **kwargs)
         self.passed = 0
-        self.skipped = []
-        self.expectedFailures = []
-        self.unexpectedSuccesses = []
-        self.tests_passed = []
+        self.testsRun = 0
+
+    @property
+    def skipped(self):
+        return [t for t in self if t.result == 'SKIPPED']
+
+    @skipped.setter
+    def skipped(self, value):
+        pass
+
+    @property
+    def expectedFailures(self):
+        return [t for t in self if t.result == 'KNOWN-FAIL']
+
+    @expectedFailures.setter
+    def expectedFailures(self, value):
+        pass
+
+    @property
+    def unexpectedSuccesses(self):
+        return [t for t in self if t.result == 'UNEXPECTED-PASS']
+
+    @unexpectedSuccesses.setter
+    def unexpectedSuccesses(self, value):
+        pass
+
+    @property
+    def tests_passed(self):
+        return [t for t in self if t.result == 'PASS']
+
+    @property
+    def errors(self):
+        return [t for t in self if t.result == 'ERROR']
+
+    @errors.setter
+    def errors(self, value):
+        pass
+
+    @property
+    def failures(self):
+        return [t for t in self if t.result == 'UNEXPECTED-FAIL']
+
+    @failures.setter
+    def failures(self, value):
+        pass
+
+    def addError(self, test, err):
+        self.add_result(test, output=self._exc_info_to_string(err, test), result_actual='ERROR')
+        self._mirrorOutput = True
+        if self.showAll:
+            self.stream.writeln("ERROR")
+        elif self.dots:
+            self.stream.write('E')
+            self.stream.flush()
+
+    def addFailure(self, test, err):
+        self.add_result(test, output=self._exc_info_to_string(err, test), result_actual='UNEXPECTED-FAIL')
+        self._mirrorOutput = True
+        if self.showAll:
+            self.stream.writeln("FAIL")
+        elif self.dots:
+            self.stream.write('F')
+            self.stream.flush()
 
     def addSuccess(self, test):
-        super(MarionetteTestResult, self).addSuccess(test)
         self.passed += 1
-        self.tests_passed.append(test)
+        self.add_result(test, result_actual='PASS')
+        if self.showAll:
+            self.stream.writeln("ok")
+        elif self.dots:
+            self.stream.write('.')
+            self.stream.flush()
 
     def addExpectedFailure(self, test, err):
         """Called when an expected failure/error occured."""
-        self.expectedFailures.append(
-            (test, self._exc_info_to_string(err, test)))
+        self.add_result(test, output=self._exc_info_to_string(err, test),
+                        result_actual='KNOWN-FAIL')
         if self.showAll:
             self.stream.writeln("expected failure")
         elif self.dots:
@@ -50,7 +115,7 @@ class MarionetteTestResult(unittest._TextTestResult):
 
     def addUnexpectedSuccess(self, test):
         """Called when a test was expected to fail, but succeed."""
-        self.unexpectedSuccesses.append(test)
+        self.add_result(test, result_actual='UNEXPECTED-PASS')
         if self.showAll:
             self.stream.writeln("unexpected success")
         elif self.dots:
@@ -58,7 +123,7 @@ class MarionetteTestResult(unittest._TextTestResult):
             self.stream.flush()
 
     def addSkip(self, test, reason):
-        self.skipped.append((test, reason))
+        self.add_result(test, output=reason, result_actual='SKIPPED')
         if self.showAll:
             self.stream.writeln("skipped {0!r}".format(reason))
         elif self.dots:
@@ -91,31 +156,30 @@ class MarionetteTestResult(unittest._TextTestResult):
                         break
                 if skip_log:
                     return
-                self.stream.writeln('START LOG:')
+                self.stream.writeln('\nSTART LOG:')
                 for line in testcase.loglines:
                     self.stream.writeln(' '.join(line).encode('ascii', 'replace'))
                 self.stream.writeln('END LOG:')
 
     def printErrorList(self, flavour, errors):
         for error in errors:
-            test, err = error[:2]
+            err = error.output
             self.stream.writeln(self.separator1)
-            self.stream.writeln("%s: %s" % (flavour, self.getDescription(test)))
+            self.stream.writeln("%s: %s" % (flavour, error.description))
             self.stream.writeln(self.separator2)
-            errlines = err.strip().split('\n')
             lastline = None
             fail_present = None
-            for line in errlines:
+            for line in err:
                 if not line.startswith('\t'):
                     lastline = line
                 if 'TEST-UNEXPECTED-FAIL' in line:
                     fail_present = True
-            for line in errlines:
+            for line in err:
                 if line != lastline or fail_present:
                     self.stream.writeln("%s" % line)
                 else:
                     self.stream.writeln("TEST-UNEXPECTED-FAIL | %s | %s" %
-                                        (self.getInfo(test), line))
+                                        (self.getInfo(error), line))
 
     def stopTest(self, *args, **kwargs):
         unittest._TextTestResult.stopTest(self, *args, **kwargs)
@@ -157,14 +221,15 @@ class MarionetteTextTestRunner(unittest.TextTestRunner):
             if stopTestRun is not None:
                 stopTestRun()
         stopTime = time.time()
-        timeTaken = stopTime - startTime
-        result.printErrors()
+        if hasattr(result, 'time_taken'):
+            result.time_taken = stopTime - startTime
         result.printLogs(test)
+        result.printErrors()
         if hasattr(result, 'separator2'):
             self.stream.writeln(result.separator2)
         run = result.testsRun
         self.stream.writeln("Ran %d test%s in %.3fs" %
-                            (run, run != 1 and "s" or "", timeTaken))
+                            (run, run != 1 and "s" or "", result.time_taken))
         self.stream.writeln()
 
         expectedFails = unexpectedSuccesses = skipped = 0
@@ -210,7 +275,8 @@ class MarionetteTestRunner(object):
                  revision=None, logger=None, testgroup="marionette", noWindow=False,
                  logcat_dir=None, xml_output=None, repeat=0, gecko_path=None,
                  testvars=None, tree=None, type=None, device_serial=None,
-                 symbols_path=None, timeout=None, es_servers=None, **kwargs):
+                 symbols_path=None, timeout=None, es_servers=None, shuffle=False,
+                 sdcard=None, **kwargs):
         self.address = address
         self.emulator = emulator
         self.emulatorBinary = emulatorBinary
@@ -244,6 +310,8 @@ class MarionetteTestRunner(object):
         self._capabilities = None
         self._appName = None
         self.es_servers = es_servers
+        self.shuffle = shuffle
+        self.sdcard = sdcard
 
         if testvars:
             if not os.path.exists(testvars):
@@ -328,7 +396,8 @@ class MarionetteTestRunner(object):
                                          bin=self.bin,
                                          profile=self.profile,
                                          baseurl=self.baseurl,
-                                         timeout=self.timeout)
+                                         timeout=self.timeout,
+                                         device_serial=self.device_serial)
         elif self.address:
             host, port = self.address.split(':')
             try:
@@ -365,7 +434,8 @@ class MarionetteTestRunner(object):
                                          logcat_dir=self.logcat_dir,
                                          gecko_path=self.gecko_path,
                                          symbols_path=self.symbols_path,
-                                         timeout=self.timeout)
+                                         timeout=self.timeout,
+                                         sdcard=self.sdcard)
         else:
             raise Exception("must specify binary, address or emulator")
 
@@ -417,6 +487,9 @@ class MarionetteTestRunner(object):
         self.reset_test_stats()
         starttime = datetime.utcnow()
         while self.repeat >=0:
+            self.logger.info('\nROUND %d\n-------' % self.repeat)
+            if self.shuffle:
+                random.shuffle(tests)
             for test in tests:
                 self.run_test(test)
             self.repeat -= 1
@@ -463,6 +536,8 @@ class MarionetteTestRunner(object):
 
         if os.path.isdir(filepath):
             for root, dirs, files in os.walk(filepath):
+                if self.shuffle:
+                    random.shuffle(files)
                 for filename in files:
                     if ((filename.startswith('test_') or filename.startswith('browser_')) and
                         (filename.endswith('.py') or filename.endswith('.js'))):
@@ -505,7 +580,10 @@ class MarionetteTestRunner(object):
                                   self.appName))
                 self.todo += 1
 
-            for i in manifest.get(tests=manifest_tests, **testargs):
+            target_tests = manifest.get(tests=manifest_tests, **testargs)
+            if self.shuffle:
+                random.shuffle(target_tests)
+            for i in target_tests:
                 self.run_test(i["path"])
                 if self.marionette.check_for_crash():
                     return
@@ -535,7 +613,7 @@ class MarionetteTestRunner(object):
                 self.todo += len(results.skipped)
             self.passed += results.passed
             for failure in results.failures + results.errors:
-                self.failures.append((results.getInfo(failure[0]), failure[1], 'TEST-UNEXPECTED-FAIL'))
+                self.failures.append((results.getInfo(failure), failure.output, 'TEST-UNEXPECTED-FAIL'))
             if hasattr(results, 'unexpectedSuccesses'):
                 self.failed += len(results.unexpectedSuccesses)
                 for failure in results.unexpectedSuccesses:
@@ -660,6 +738,10 @@ class MarionetteTestOptions(OptionParser):
                         type='str',
                         help='set a custom resolution for the emulator'
                              'Example: "480x800"')
+        self.add_option('--sdcard',
+                        action='store',
+                        dest='sdcard',
+                        help='size of sdcard to create for the emulator')
         self.add_option('--no-window',
                         action='store_true',
                         dest='noWindow',
@@ -743,6 +825,11 @@ class MarionetteTestOptions(OptionParser):
                         dest='es_servers',
                         action='append',
                         help='the ElasticSearch server to use for autolog submission')
+        self.add_option('--shuffle',
+                        action='store_true',
+                        dest='shuffle',
+                        default=False,
+                        help='run tests in a random order')
 
     def verify_usage(self, options, tests):
         if not tests:
